/*   This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; version 2 of the License.
   
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     (C) 2006 schan
*/
#include "engine.h"
#include "defines.h"
#include "SDL/SDL.h"

// AppState //////////////////////////////////////////////////////////////////////////////////


AppState::AppState()
{
};
	
AppState::~AppState()
{
};

void AppState::init(AppEngine *_app)
{
	app = _app;
	on_init(); // call virtual function in inherited class
}
				

// AppEngine //////////////////////////////////////////////////////////////////////////////////

AppEngine::AppEngine(int _max_states)
{
	max_states = _max_states;
	state = new AppState*[max_states];
	for (int i=0; i<max_states; i++)
	{
		state[i] = 0;
	}	
}

AppEngine::~AppEngine()
{
	for (int i=0; i<max_states; i++)
	{
		if (state[i] == 0) delete state[i];
	}	
	delete state;
}
	
bool AppEngine::add_state(AppState *st, int id)
{
	if (id>=max_states) return false;
		
	state[id]=st;
	state[id]->init(this);
	return true;
}

	
void AppEngine::start(int _state)
{
	active_state = _state;
	next_state = -1;
	
	quit = false;
	first_think = true;

	SDL_Event event;
	// Main loop: loop forever.
	while (!quit)
	{
		state[active_state]->think(first_think);
		first_think=false;

		while(SDL_PollEvent(&event))
		{
			switch(event.type)
			{
				case SDL_KEYDOWN:
					switch(event.key.keysym.sym)
					{
						case SDLK_UP: 
								state[active_state]->on_button_down(GP2X_BUTTON_UP);
							break;
						case SDLK_DOWN: 
								state[active_state]->on_button_down(GP2X_BUTTON_DOWN);
							break;
						case SDLK_LEFT: 
								state[active_state]->on_button_down(GP2X_BUTTON_LEFT);
							break;
						case SDLK_RIGHT: 
								state[active_state]->on_button_down(GP2X_BUTTON_RIGHT);
							break;
						case SDLK_a:
								state[active_state]->on_button_down(GP2X_BUTTON_A);
							break;
						case SDLK_b: 
								state[active_state]->on_button_down(GP2X_BUTTON_B);
							break;
						case SDLK_x: 
								state[active_state]->on_button_down(GP2X_BUTTON_X);
							break;
						case SDLK_y: 
								state[active_state]->on_button_down(GP2X_BUTTON_Y);
							break;
						case SDLK_l: 
								state[active_state]->on_button_down(GP2X_BUTTON_L);
							break;
						case SDLK_r:
								state[active_state]->on_button_down(GP2X_BUTTON_R);
							break;
						case SDLK_RETURN: 
								state[active_state]->on_button_down(GP2X_BUTTON_START);
							break;
						case SDLK_ESCAPE: 
								state[active_state]->on_button_down(GP2X_BUTTON_SELECT);
							break;
						case SDLK_PLUS: 
								state[active_state]->on_button_down(GP2X_BUTTON_VOLUP);
							break;
						case SDLK_MINUS: 
								state[active_state]->on_button_down(GP2X_BUTTON_VOLDOWN);
							break;
					}
					break;
				case SDL_JOYBUTTONDOWN:
					state[active_state]->on_button_down(event.jbutton.button);
					break;
			}
		}
		
		
		if (next_state>-1)
		{
			active_state = next_state;
			next_state = -1;
			first_think = true;
		}
		
	}
}
	
void AppEngine::change_to_state(int _state)
{
	next_state = _state;
}
	
void AppEngine::stop()
{
	quit=true;
}
	
