/*   This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; version 2 of the License.
   
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     (C) 2006 schan
*/
#include "widget.h"
#include "gui.h"
#include "defines.h"

Gui::Gui(int _x, int _y, int _w, int _h)
{
	w = _w;
	h = _h;
	x = _x;
	y = _y;
	selected = -1;
	
	arrow_up = new Bitmap("pixmaps/arrow_up.bmp");
	arrow_down = new Bitmap("pixmaps/arrow_down.bmp");
}

Gui::~Gui()
{
	clear();
	
	delete arrow_up;
	delete arrow_down;
}

void Gui::clear()
{
	for (int i=0; i<widgets.size(); i++)
	{
		delete widgets.at(i);
	}
	widgets.clear();
	selected = -1;
}

void Gui::add_widget(Widget *w)
{
	widgets.push_back(w);

	int count = widgets.size();
	if (selected==-1 && widgets.at(count-1)->is_selectable() )
	{
		selected = count-1;	
	}
}

void Gui::draw(SDL_Surface *s)
{
	SDL_Rect clip_rect = {x,y,w,h};
	SDL_SetClipRect(s,&clip_rect);
//	SDL_FillRect(s,&clip_rect,0);
	
	int total_height = 0;
	for (int i=0; i<widgets.size(); i++)
	{
		total_height += widgets.at(i)->get_height(i==selected);
	}
	
	// widgets width
	int widgets_w = w;
	if (total_height>h)
	{
		// reduce widgets width
		widgets_w -= arrow_up->w()+2;
	}
	

	// selected's widget y
	int selected_y = -1;
	int dest_y = 0;
	for (int i=0; i<widgets.size(); i++)
	{
		if (i==selected) selected_y = dest_y;
		dest_y += widgets.at(i)->get_height(i==selected);
	}
	int scroll_y = 0;
	
	if (total_height>h)
	{
		if (selected_y>40) scroll_y=selected_y-40;
		int max_scroll_y = total_height-h;
		if (scroll_y>max_scroll_y) scroll_y = max_scroll_y;

		if (scroll_y>0)
		{
			arrow_up->draw(s,x+w-arrow_up->w(),y);
		}
		if (scroll_y<max_scroll_y)
		{
			arrow_down->draw(s,x+w-arrow_up->w(),y+h-arrow_down->h());
		}
	}

	
	// draw widgets
	dest_y = 0;
	for (int i=0; i<widgets.size(); i++)
	{
		int widget_h = widgets.at(i)->get_height(i==selected);
		int total_y = y-scroll_y+dest_y;
		if (total_y+widget_h>=y)
		{
			widgets.at(i)->draw(s, x, total_y, widgets_w, i==selected);
		}
		dest_y += widget_h;
//		if (dest_y>w) break;
	}

	
		



	SDL_SetClipRect(s,NULL);
}
		
void Gui::on_button_down(int button)
{
	if (selected>=0 && !(widgets.at(selected)->on_button_down(button)) )
	{
		// if buttons ignored... check if selection changes

		// select next
		if (button==GP2X_BUTTON_DOWN && selected<widgets.size())
		{
			int next_selectable = -1;
			int idx = selected +1;
			while (idx<widgets.size() && next_selectable==-1)
			{
				if (widgets.at(idx)->is_selectable()) next_selectable = idx;
				idx++;
			}
			if (next_selectable>=0) selected = next_selectable;
		}
		
		// select previous
		else if (button==GP2X_BUTTON_UP && selected>0)
		{
			int prev_selectable = -1;
			int idx = selected -1;
			while (idx>=0 && prev_selectable==-1)
			{
				if (widgets.at(idx)->is_selectable()) prev_selectable = idx;
				idx--;
			}
			if (prev_selectable>=0) selected = prev_selectable;
		}		
	}
}

void Gui::on_button_up(int button)
{
	if (selected>=0) widgets.at(selected)->on_button_up(button);
}

		
