/*   This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; version 2 of the License.
   
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     (C) 2006 schan
*/
#include <stdlib.h>
#include "time.h"
#include "unistd.h"
#include <iostream>
#include <fstream>
#include "SDL/SDL.h"
//#include "SDL/SDL_image.h"
#include "util.h"

#include "shared.h"
#include "history.h"

#include "defines.h"

#include "bitmap.h"
#include "bitmapfont.h"
#include "text.h"

#include "gui_skin.h"

#include "state_selector.h"
#include "state_menu.h"
#include "state_loading.h"
#include "state_reader.h"
#include "state_preferences.h"
#include "state_choosefont.h"
#include "state_history.h"

int main(int argc, char *argv[])
{	
	
	
	// important inits ------------------------------------------------------------------------

	util::hw_set_cpu(LOW_SPEED);

	srand(time(0));

	if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_JOYSTICK) < 0)
	{
		fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
		return -1;
	}

	// load minimum resources -----------------------------------------------------------------

	shared::background_image = new Bitmap("pixmaps/bg.bmp");

	// continue init... -----------------------------------------------------------------
	
	shared::screen = SDL_SetVideoMode(320, 240, 16, SDL_HWSURFACE | SDL_DOUBLEBUF);
	SDL_ShowCursor(false);
	
	// show an image before loading data
	shared::background_image->draw(shared::screen,0,0);
	SDL_Flip(shared::screen);

	// joystick
	shared::joystick = SDL_JoystickOpen(0);

	// options
	shared::options = new Options();
	shared::options->load();	

	// font
	shared::text_font = new BitmapFont();

	// text & renderer		
	shared::plain_text_renderer = new PlainTextRenderer(shared::options);	
	shared::text = new Text();
	shared::text->set_renderer(shared::plain_text_renderer);
	shared::text->set_font(shared::text_font);
	shared::text->set_viewport_width(shared::SCREEN_W);
	shared::text->set_viewport_height(shared::SCREEN_H);
	
	// history
	shared::history = new History();
	shared::history->load();

	// other resources
	GuiSkin::load_big_font("pixmaps/gui_big.bmp",CHAR_LIST);
	GuiSkin::load_small_font("pixmaps/gui_small.bmp",CHAR_LIST_GUI);
	GuiSkin::load_check_on("pixmaps/check_on.bmp");
	GuiSkin::load_check_off("pixmaps/check_off.bmp");
	
	std::ifstream color_file("pixmaps/gui_color.cfg");
	int c_r,c_g,c_b;
    color_file >> c_r;
    color_file >> c_g;
    color_file >> c_b;
	color_file.close();	
	SDL_Color gui_color = {c_r, c_g, c_b};	
	GuiSkin::set_selected_color(gui_color);
	

	// main loop ------------------------------------------------------------------------------

	shared::app = new AppEngine(10);
	shared::app->add_state(new MenuState(), ST_MENU);
	shared::app->add_state(new SelectorState(), ST_SELECTOR);
	shared::app->add_state(new LoadingState(), ST_LOADING);
	shared::app->add_state(new ReaderState(), ST_READER);
	shared::app->add_state(new PreferencesState(), ST_PREFERENCES);
	shared::app->add_state(new ChooseFontState(), ST_CHOOSEFONT);
	shared::app->add_state(new HistoryState(), ST_HISTORY);

	shared::app->start(ST_MENU);

	
	// cleanup --------------------------------------------------------------------------------------
	delete shared::app;
	delete shared::text_font;
	
	delete shared::plain_text_renderer;
	delete shared::options;
	delete shared::history;
	delete shared::background_image;
	
	chdir("/usr/gp2x");
	execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);
	return 0;
	
}
