/*   This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; version 2 of the License.
   
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     (C) 2006 schan
*/
#include "state_reader.h"
#include "defines.h"
#include "bitmap.h"
#include "SDL/SDL.h"
#include "shared.h"
#include "util.h"
#include "gui.h"
#include "gui_skin.h"

ReaderState::~ReaderState()
{
}

void ReaderState::on_init()
{
};
		
void ReaderState::on_button_down(int button)
{
	if (button==GP2X_BUTTON_START)
	{
		// save file to history
		shared::history->add(shared::text->get_filename(), shared::text->get_percent());
		
		// go to menu
		app->change_to_state(ST_MENU);
	}
	if (button==GP2X_BUTTON_X)
	{
		shared::options->set_showstatus(!shared::options->get_showstatus());
		shared::text->set_viewport_height(shared::options->get_showstatus() ? 228 : 240 );
		redraw();
	}
};

void ReaderState::on_button_up(int button)
{
};
	
void ReaderState::think(bool first)
{
	if (first)
	{	
		timer = 0;
		last_tick = SDL_GetTicks();
		shared::text->set_viewport_height(shared::options->get_showstatus() ? 228 : 240 );
		redraw();
	}
	
	Uint8 *keystate = SDL_GetKeyState(NULL);
	if (SDL_JoystickGetButton(shared::joystick,GP2X_BUTTON_DOWN) || keystate[SDLK_DOWN] )
	{
		if (last_tick+timer<=SDL_GetTicks())
		{
			shared::text->next_line();
			redraw();
			last_tick = SDL_GetTicks();
			if (timer==0) timer=200;
			else timer = timer-5;
			if (timer<10) timer=10;
		}
	}
	else if (SDL_JoystickGetButton(shared::joystick,GP2X_BUTTON_UP) || keystate[SDLK_UP] )
	{
		if (last_tick+timer<=SDL_GetTicks())
		{
			shared::text->prev_line();
			redraw();
			last_tick = SDL_GetTicks();
			if (timer==0) timer=200;
			else timer = timer-5;
			if (timer<10) timer=10;
		}
	}
	else if (SDL_JoystickGetButton(shared::joystick,GP2X_BUTTON_R) ||
	         SDL_JoystickGetButton(shared::joystick,GP2X_BUTTON_L) || 
	         SDL_JoystickGetButton(shared::joystick,GP2X_BUTTON_RIGHT) || 
	         keystate[SDLK_r] )
	{
		if (last_tick+timer<=SDL_GetTicks())
		{
			shared::text->next_page();
			redraw();
			last_tick = SDL_GetTicks();
			if (timer==0) timer=200;
			else timer = timer-5;
			if (timer<10) timer=10;
		}
	}
	else if (SDL_JoystickGetButton(shared::joystick,GP2X_BUTTON_LEFT) ||  
	         keystate[SDLK_l] )
	{
		if (last_tick+timer<=SDL_GetTicks())
		{
			shared::text->prev_page();
			redraw();
			last_tick = SDL_GetTicks();
			if (timer==0) timer=200;
			else timer = timer-5;
			if (timer<10) timer=10;
		}
	}
	else timer = 0;
}

void ReaderState::redraw()
{
	shared::text->render(shared::screen);

	if (shared::options->get_showstatus())
	{
		SDL_Rect r = {0,228,320,12};
		SDL_Color c = GuiSkin::get_small_font()->get_bg_color();
		SDL_FillRect(shared::screen, &r, SDL_MapRGB(shared::screen->format, c.r, c.g, c.b));

		GuiSkin::get_small_font()->draw_text(shared::text->get_filename(),shared::screen, 50, 228);

		char buffer[10];	
		sprintf(buffer,"%2.2f%%",shared::text->get_percent());
		GuiSkin::get_small_font()->draw_text(buffer,shared::screen, 0, 228);

	}
		
	SDL_Flip(shared::screen);
	
};
