/*   This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; version 2 of the License.
   
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     (C) 2006 schan
*/
#ifndef TEXT_H
#define TEXT_H

#include "SDL/SDL.h"
#include <string>
#include "bitmapfont.h"

class TextRenderer
{
	public:
		virtual ~TextRenderer();
		
		virtual void reset() = 0;
		virtual void add_line(std::string line, int encoding) = 0;

		virtual void set_font(BitmapFont *f) = 0; // reload after setting font!!

		virtual void set_viewport_width(int w) = 0;  // reload after setting width!!
		virtual void set_viewport_height(int h) = 0; // no need to reload after setting height

		virtual void render(SDL_Surface *dest) = 0;
		
		virtual void next_line() = 0;
		virtual void prev_line() = 0;
		virtual void next_page() = 0;
		virtual void prev_page() = 0;
		virtual void go_to_line(unsigned long int n) = 0;
		virtual void go_to_percent(float p) = 0;
		virtual float get_percent() = 0;
};

class Text
{
	public:
		Text();		
		~Text();
		
		void set_renderer(TextRenderer *r);
		void set_file(const std::string &filename, int encoding);		
		const char *get_filename() { return filename.c_str(); };
		const std::string get_filename_string() { return filename; };

		void load();
		void reload();

		void render(SDL_Surface *dest);
		void next_line();
		void prev_line();
		void next_page();
		void prev_page();
		void go_to_line(unsigned long int n);
		inline void go_to_percent(float p) { renderer->go_to_percent(p); }
		inline float get_percent() { return renderer->get_percent(); }
		
		void set_font(BitmapFont *f);
		void set_viewport_width(int w);
		void set_viewport_height(int h);


	private:
		TextRenderer *renderer;
		std::string filename;
		BitmapFont *font;
		int width;
		int height;
		int encoding;
};

#endif
